function h_err = errstep(err,err_old,h,h_old,ord)
%
% function h_err = errstep(err,err_old,h,h_old,ord)
%
% Estimate a new step size via a Gustafsson approach.
%
% More details can be found:
% E. Hairer and G. Wanner, Solving Ordinary Differential Equations II,
% Stiff and Differential-Algebraic Problems, Springer-Verlag, Berlin,
% 1991, pp. 31.
% or their RADAU5 code.

% This file is part of the exp4 package available at
% http://na.uni-tuebingen.de.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% file COPYING of the distribution or send email to
% exp4@na.uni-tuebingen.de.

if (err<=1) & (h_old>0)
  Gus = gustafsson_fac(err, err_old, h, h_old, ord);
  Class = classical_fac(err, ord);
  h_err = h*clip_fac(min(Gus,Class));
else
  h_err = h*clip_fac(classical_fac(err, ord));
end
  
function [f_gus] = gustafsson_fac(err, err_old, h, h_old, ord)
  
f_gus = (h/h_old)*(err_old/(err^2))^(1/ord);
  
function [f_cla] = classical_fac(err, ord)
  
f_cla = err^(-1/ord);
  
function [f] = clip_fac(f)

facmin = 0.2;
facmax = 5.0;
fac    = 0.9;

f = max(min(fac*f,facmax),facmin);