% This is a helper script for EXP4 and EXP4EX.
% Do not run this script on its own.

% This file is part of the exp4 package available at
% http://na.uni-tuebingen.de.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% file COPYING of the distribution or send email to
% exp4@na.uni-tuebingen.de.

% --------------------------------------------------------------------------  
% Check user-input and initialize the core integrators
% --------------------------------------------------------------------------  
 
% find out which integrator runs ("to krylov or not to krylov")
isExp4Flag = strcmp(IntegratorName,'exp4');

if ~(isExp4Flag | strcmp(IntegratorName,'exp4ex'))
  help exp4_initialize
  error('Unkown integrator. Sigh!')
end

if nargin < 4
  eval(['help ',IntegratorName]);
  error('Four arguments expected.')
end

EXP4_DATA = struct(...
    'Name',odefile,...
    'VarArg',{varargin},...
    'JConstant',strcmp(odeget(options,'JConstant','off'),'on'),...
    'JIsMatrix',strcmp(odeget(options,'Jacobian','off'),'on'),...
    'Jacobian',[],...
    'JacobianEvalPoint',[],...
    'Neq',length(y),...
    'SqrtNeq',sqrt(length(y)),...
    'Stats',strcmp(odeget(options,'Stats','off'),'on'),...
    'Scale',[],...
    'Use2Norm',0,...
    'Outflag',0,...
    'KrylovTestIndex',[1,2,3,4,6,8,11,15,20,27,36,46,57,70,85,100],...
    'Threshold',[],...
    'Atol',[], ...
    'Rtol',[]);
  
EXP4_STATISTICS = struct(...
    'Odefile',odefile,...
    'Interval',[],...
    'AtolAndRtol',[],...
    'Time',cputime,...
    'Flops',flops,...
    'SelfFlops',flops,...
    'UserFlops',flops,...
    'nFuncEvals',0,...
    'nNewJac',0,...
    'nJacv',0,...
    'nJacTv',0,...
    'nSteps',0,...
    'nFails',0,...
    'nReject',0,...
    'nKrylov1',[0 0],...
    'nKrylov2',[0 0],...
    'nKrylov3',[0 0],...
    'KrylovType1',[],...
    'KrylovType2',[],...
    'KrylovType3',[]);

vprint(['---------------------------------------------------------------\n',...
        '          Exponential method of order 4(3,2) (W)\n',...
        '---------------------------------------------------------------\n']);

if isExp4Flag
  vprint('Compute phi(z) with Krylov techniques.\n');
else
  vprint('Compute phi(z) directly.\n')
end

if isfield(options,'Use2Norm')
  EXP4_DATA.Use2Norm = strcmp(options.Use2Norm,'on');
end

% setup the Krylov specific options
if isExp4Flag
  if isfield(options,'Krylov')
    if exist(options.Krylov) == 2
      Krylov1 = options.Krylov;
    else
      Krylov1 = 'phi_arn';
      fprintf('Could not find %s.m as implementation of the ',options.Krylov);
      fprintf('Krylov process.\n');
      fprintf('Using Arnoldi process as default method :-)\n');
    end
  elseif isfield(options,'Krylov1')
    if exist(options.Krylov1) == 2
      Krylov1 = options.Krylov1;
    else
      Krylov1 = 'phi_arn';
      fprintf('Could not find %s.m as implementation of the ',options.Krylov1);
      fprintf('Krylov process.\n');
      fprintf('Using Arnoldi process as default method :-)\n');
    end
  else
    Krylov1 = 'phi_arn';
  end
  
  if isfield(options,'Krylov2')
    if exist(options.Krylov2) == 2
      Krylov2 = options.Krylov2;
    else
      Krylov2 = 'phi_arn';
      fprintf('Could not find %s.m as implementation of the ',options.Krylov2);
      fprintf('Krylov process.\n');
      fprintf('Using Arnoldi process as default method :-)\n');
    end
  else
    Krylov2 = 'phi_arn';
  end
  
  if isfield(options,'Krylov3')
    if exist(options.Krylov3) == 2
      Krylov3 = options.Krylov3;
    else
      Krylov3 = 'phi_arn';
      fprintf('Could not find %s.m as implementation of the ',options.Krylov3);
      fprintf('Krylov process.\n');
      fprintf('Using Arnoldi process as default method :-)\n');
    end
  else
    Krylov3 = 'phi_arn';
  end
  
  if ~isempty(findstr('lan',Krylov1))
    EXP4_STATISTICS.KrylovType1 = 'Lanczos process';
  elseif ~isempty(findstr('cgs',Krylov1))
    EXP4_STATISTICS.KrylovType1 = 'transpose-free Lanczos process';
    switch2_thres = 6;
  else
    EXP4_STATISTICS.KrylovType1 = 'Arnoldi process';
  end
  if ~isempty(findstr('lan',Krylov2))
    EXP4_STATISTICS.KrylovType2 = 'Lanczos process';
    switch2_thres = 6;
  elseif ~isempty(findstr('cgs',Krylov2))
    EXP4_STATISTICS.KrylovType2 = 'transpose-free Lanczos process';
    switch2_thres = 8;
  else
    EXP4_STATISTICS.KrylovType2 = 'Arnoldi process';
  end
  if ~isempty(findstr('lan',Krylov3))
    EXP4_STATISTICS.KrylovType3 = 'Lanczos process';
    switch3_thres = 6;
  elseif ~isempty(findstr('cgs',Krylov3))
    EXP4_STATISTICS.KrylovType3 = 'transpose-free Lanczos process';
    switch3_thres = 8;
  else
    EXP4_STATISTICS.KrylovType3 = 'Arnoldi process';
  end
  
  KrylovMax1 = 36; % max. # of Krylov steps in the first stage
  if isfield(options,'KrylovMax1')
    if ~isempty(options.KrylovMax1)
      KrylovMax1 = options.KrylovMax1;
    end
  end
  
  KrylovMax2 = 36; % max. # of Krylov steps in the second and third stage
  if isfield(options,'KrylovMax2')
    if ~isempty(options.KrylovMax2)
      KrylovMax2 = options.KrylovMax2;
    end
  end
  
  KrylovMax3 = 36; % max. # of Krylov steps in the second and third stage
  if isfield(options,'KrylovMax3')
    if ~isempty(options.KrylovMax3)
      KrylovMax3 = options.KrylovMax3;
    end
  end
  
  vprint('First stage:  %s, with at most %3.0f steps\n',...
      EXP4_STATISTICS.KrylovType1,KrylovMax1);
  vprint('Second stage: %s, with at most %3.0f steps\n',...
      EXP4_STATISTICS.KrylovType2,KrylovMax2);
  vprint('Third stage:  %s, with at most %3.0f steps\n',...
      EXP4_STATISTICS.KrylovType3,KrylovMax3);
  
  k_guess_hist = 3;
  if isfield(options,'kGuessHist')
    if ~isempty(options.kGuessHist)
      k_guess_hist = options.kGuessHist;
    end
  end
  
  if ~isempty(findstr('cgs',Krylov1))
    vprint('Setting k_guess to %2.0f\n',k_guess_hist);
  end

  % initialize Optimal step, Good step, ...
  KrylovHistory = krystep([KrylovMax1 KrylovMax2 KrylovMax3]);
  
  k_guess = zeros(1,max(k_guess_hist,1));
  switch2 = 0; %switch2_thres = 6;
  switch3 = 0; %switch3_thres = 6;  
  
end % end of Krylov specific initialization

% -------------------------------------------------------------------------- 
% The coefficients of the method
% -------------------------------------------------------------------------- 

Gamma4 = 1/3;

% beta   = [ -5/6 0  1  1   -4/3    1  1/6 ];
% hbetaw = [ -1   2  0 -1      0    0    1 ];
% hbeta3 = [  1/6 0  1 -1/2 -2/3  1/2  1/2 ];

alpha4 = [ -7/300    97/150 -37/300 ];
alpha7 = [ -541/300 -7/75    269/300 2/3 2/3 2/3 ];
a4     = sum(alpha4);
a7     = sum(alpha7);
  
alpha7(1) = alpha7(1) + sum(alpha7(4:6));
% beta(1)   = beta(1) + sum(beta(4:7));  
% hbetaw(1) = hbetaw(1) + sum(hbetaw(4:7));  
% hbeta3(1) = hbeta3(1) + sum(hbeta3(4:7));  

order = 3;

% the dense output coefficients
dense = [ 1   -2    1    0;
          0    2   -2    0;
	  0    0    1    0;
	  0    2   -4/3  1/3;
	  0    0    0   -4/3;
	  0    0    0    1;
	  0   -1/2  2/3  0   ];

% --------------------------------------------------------------------------  
% Further checks on the user-input.
% --------------------------------------------------------------------------  

% Test that tspan is internally consistent.
tspan = tspan(:);
ntspan = length(tspan);
if ntspan == 1
  t = 0;
else
  t = tspan(1);
end
tfinal = tspan(ntspan);
if t == tfinal
  error('The last entry in tspan must be different from the first entry.');
end
tdir = sign(tfinal - t);
if any(tdir * (tspan(2:ntspan) - tspan(1:ntspan-1)) <= 0)
  error('The entries in tspan must strictly increase or decrease.');
end

EXP4_STATISTICS.Interval = [t tfinal];

% Get options, and set defaults for Atol and Rtol.
EXP4_STATISTICS.AtolAndRtol = ...
    [min(odeget(options,'Abstol',1e-6)) max(odeget(options,'Abstol',1e-6)) ...
     min(odeget(options,'RelTol',1e-3)) max(odeget(options,'RelTol',1e-3))];
sc_norm('init',options);
sc_norm('set',y);

% Get options, and set defaults for hmax.
hmax = abs(odeget(options,'MaxStep',tfinal-t));
 
% Allocate memory if we're generating output.
if 0<nargout
  if ntspan == 1                        % only 1 output at tfinal
    nout = 0;
  else
    if ntspan > 2                       % output only at tspan points
      tout = zeros(ntspan,1);
      yout = zeros(ntspan,EXP4_DATA.Neq);
    else                                % alloc in chunks
      chunk = ceil(128 / EXP4_DATA.Neq);
      tout = zeros(chunk,1);
      yout = zeros(chunk,EXP4_DATA.Neq);
    end
    tout(1) = t;
    yout(1,:) = y.';
    nout = 1;
  end
end

% Set the output flag.
if (ntspan > 2) & (nargout == 2)
  EXP4_DATA.Outflag = 1;                % output only at tspan points
elseif (ntspan == 2) & (nargout == 2)
  EXP4_DATA.Outflag = 2;                % computed points
else 
  EXP4_DATA.Outflag = 0;                % no output, write to screen
  EXP4_DATA.Stats = 1;
end

% check matlab version
% if sscanf(version,'%f',[1 1])>=5.2
%   EXP4_DATA.FancyFeatureSupportFlag = 1; % :-)
% end
    
% This code does not support all options of the matlab ODE solvers
UnsupportedOptions = {'BDF','Events','JPattern','Mass',...
      'MassConstant','MaxOrder','OutputFcn','OutputSel',...
      'Refine','Vectorized','NormControl'};
for k = 1:length(UnsupportedOptions)
  if ~isempty(odeget(options,UnsupportedOptions{k}))
    error([upper(IntegratorName) ' does not support the option ' ...
	  UnsupportedOptions{k} '.'])
  end
end

% -------------------------------------------------------------------------- 
% constants for step size selection etc.
% -------------------------------------------------------------------------- 
facmin_kry = 1/3;
reject = 0; fail = 0;
reuse_J = 0; old_J_flag = 1;
reuse_k = 0; reuse_k_counter = 0; max_reuse_k = 15;
errw = 1; err3 = 1; err = 1; err_acc = 1; h_acc = 0;

k = zeros(EXP4_DATA.Neq,7);
V = []; H = [];

% the initial step size
h = odeget(options,'InitialStep');
if isempty(h)
  [h,fy] = h_init(tspan,y,4); 
else
  fy = odefun('YDot',y);
end

h = min([h,hmax,abs(t-tfinal)]);
h_old = h;

% clear temporary variables
IntegratorName = [];
isExp4Flag = [];
UnsupportedOptions = [];
