function [z,h] = kry_step_red(H,h,scaled_norm,norm_v,errw,res,z,Gamma4,fac_t)
%
% function [z,h] = kry_step_red(H,h,scaled_norm,norm_v,errw,res,z,Gamma4,fac_t)
%
% Helper function for phi_*.m.
% Reduce the step size h until the already computed Krylov basis
% suffices for convergence.

% This file is part of the exp4 package available at
% http://na.uni-tuebingen.de.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% file COPYING of the distribution or send email to
% exp4@na.uni-tuebingen.de.

res_bound = 1/max(1,errw);
[k,dummy] = size(z);
h_org = h;

while res >= res_bound
  res_mult = scaled_norm*Gamma4*h*fac_t;
  fprintf('failed residual %e\n',res)
  
  % check if 2/3 h does the job
  red_fact = 2/3;   
  if red_fact*h*res_mult*abs(z(k,2)) >= res_bound
    red_fact = 1/3;
    % check if 1/3 h does the job
    if red_fact*h*res_mult*abs(z(k,1)) >= res_bound
      red_fact = 1/6;
    end
  end
  
  h = h*red_fact;
  fprintf('1st residual %e\n',res_mult*h*abs(z(k,1)))
  fprintf('2nd residual %e\n',res_mult*h*abs(z(k,2)))
  
  z = phim(H(1:k,1:k),h*Gamma4,fac_t)*norm_v;
  
  res = scaled_norm*h*h*Gamma4*fac_t*abs(z(k,fac_t));
  fprintf('######### step size reduction by Krylov-Process: %g, %g, %g\n',...
      h_org/h,res,h);
  
end % while
