function out = krystep(steps,History)
%
% function out = krystep(steps,History)
%
% Called with only one argument KRYSTEP sets up the Krylov window etc.
% Called with two arguments KRYSTEP computes the factor for the step
% size depending on the "Krylov history".

% This file is part of the exp4 package available at
% http://na.uni-tuebingen.de.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% file COPYING of the distribution or send email to
% exp4@na.uni-tuebingen.de.

global EXP4_DATA EXP4_STATISTICS
  
if nargin==1 % steps is maximal # Krylov steps
%      'num1',[1,1,1],...
  out = struct(...
      'num1',[10,10,10],...
      'num2',[10,10,10],...
      'num3',[10,10,10],...
      'Exponent',[1/3,1/3,1/3],...
      'Optimal',[27,27,27],...
      'Good',[20,20,20]);
  switch EXP4_STATISTICS.KrylovType1
    case 'Lanczos process',
      out.Exponent(1) = 1/2;
    case 'transpose-free Lanczos process',
      out.Exponent(1) = 1/2;
    case 'Arnoldi process',
      out.Exponent(1) = 1/3;
  end  
%   switch EXP4_STATISTICS.KrylovType23
%     case 'Lanczos process',
%       out.Exponent(2:3) = 1/2;
%     case 'transpose-free Lanczos process',
%       out.Exponent(2:3) = 1/2;
%     case 'Arnoldi process',
%       out.Exponent(2:3) = 1/3;
%   end  
  switch EXP4_STATISTICS.KrylovType2
    case 'Lanczos process',
      out.Exponent(2) = 1/2;
    case 'transpose-free Lanczos process',
      out.Exponent(2) = 1/2;
    case 'Arnoldi process',
      out.Exponent(2) = 1/3;
  end  
  switch EXP4_STATISTICS.KrylovType3
    case 'Lanczos process',
      out.Exponent(3) = 1/2;
    case 'transpose-free Lanczos process',
      out.Exponent(3) = 1/2;
    case 'Arnoldi process',
      out.Exponent(3) = 1/3;
  end  

  for k=1:3
    [dummy, index] = min(abs(EXP4_DATA.KrylovTestIndex - (3/4)*steps(k)));
    out.Optimal(k) = EXP4_DATA.KrylovTestIndex(index);
    [dummy, index] = min(abs(EXP4_DATA.KrylovTestIndex - (4/7)*steps(k)));
    out.Good(k)    = EXP4_DATA.KrylovTestIndex(index);
  end
  
else % compute factor depending on Krylov window
  out = ones(1,3);
  for k=1:3
    if max(steps(k),History.num1(k))<History.Good(k)
      out(k) = (History.Optimal(k)/max([1,steps(k),...
	    History.num1(k)]))^History.Exponent(k);
      if max([steps(k),History.num1(k),History.num2(k)])<4
	out(k) = 4;
	if History.num3(k)<4
	  out(k) = 8;
	end
      end
    end
  end
  out = min(out);
end

