function M = plotbruss2d(y,t,index,varargin)
% 
% function M = plotbruss2d(y,t,index,varargin)
%
% Plot the results of the BRUSS2D example.
%
%   y: the solution of the Brusselator example.
%   t: the time grid of the solution.
%   index: plot the solution at the time grid t(index).
%   An optional fourth argument sets OpenGL as renderer.
%   
% See also BRUSS2D.

% This file is part of the exp4 package available at
% http://na.uni-tuebingen.de.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% file COPYING of the distribution or send email to
% exp4@na.uni-tuebingen.de.

[M,N] = size(y); 
N = sqrt(N/2);
y = real(y);

ort = linspace(0,1,N);

h = figure;
subplot(1,2,1);
axis([0 1 0 1 -6 8]);
set(gca,'nextplot','replacechildren');
subplot(1,2,2);
axis([0 1 0 1 -1 5]);
set(gca,'nextplot','replacechildren');

tmp = get(gcf,'position');
winsize = tmp(1:2);
tmp = get(0,'ScreenSize');
winsize(3) = tmp(3)*2/3;
winsize(4) = tmp(4)*1/3;
set(gcf,'position',winsize);

if nargin>3
  set(h,'renderer','opengl')
end

M = moviein(length(index));

for k = 1:length(index)
  s = sprintf('%9.2e',t(index(k)));
  
  subplot(1,2,1);
  comp = 1;
  surfl(ort,ort,reshape(y(index(k),1+(comp-1)*N^2:comp*N^2),N,N));
  colormap('bone');
  shading('interp');
  title(['u(' s ')']);

  subplot(1,2,2);
  comp = 2;
  surfl(ort,ort,reshape(y(index(k),1+(comp-1)*N^2:comp*N^2),N,N));         
  colormap('bone');
  shading('interp');
  title(['v(' s ')']);

  M(:,k) = getframe;
end

movie(h,M,1);
