function nrm = sc_norm(flag,varargin)
%
% function nrm = sc_norm(flag,varargin)
%
% Compute a weighted norm of the vector y.
% || y || = sqrt(sum(j=1..dim,(y(j)/scale(j))^2)/dim)
%
% If SCALE is supplied use this scale for the weighted norm otherwise
% the scale in EXP4_DATA.Scale.
% If EXP4_DATA.Use2Norm is set a weighted 2-Norm is used otherwise
% a weighted inf-norm.

% This file is part of the exp4 package available at
% http://na.uni-tuebingen.de.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% file COPYING of the distribution or send email to
% exp4@na.uni-tuebingen.de.

global EXP4_DATA

switch flag
  case 'init'
    Rtol = odeget(varargin{1},'RelTol',1e-3);
    Rtol = Rtol(:);
    Atol = odeget(varargin{1},'Abstol',1e-6);
    Atol = Atol(:);
    if any(Rtol < 0) | any(Atol < 0)
      error('Tolerance RelTol and all entries of AbsTol must be positive.');
    end
    if any(Rtol < 100*eps)
      warning('RelTol is smaller then 100*eps');
    end
    if length(Atol) ~= 1 & length(Atol) ~= EXP4_DATA.Neq
      error(['Vector abstol must be same length as y vector ('...
	     num2str(EXP4_DATA.Neq) ').']);
    end
    if length(Rtol) ~= 1 & length(Rtol) ~= EXP4_DATA.Neq
      error(['Vector reltol must be same length as y vector ('...
	     num2str(EXP4_DATA.Neq) ').']);
    end
    
    EXP4_DATA.Rtol = Rtol;
    EXP4_DATA.Atol = Atol;
    if length(EXP4_DATA.Rtol) == 1 & EXP4_DATA.Rtol ~= 0
      EXP4_DATA.Threshold = Atol/Rtol;
    end
    
  case 'set'
    EXP4_DATA.Scale=getscale(varargin{:});
  
  case 'norm'
    if nargin>2
      nrm = current_norm(varargin{1}./getscale(varargin{2}));
    else
      nrm = current_norm(varargin{1}./EXP4_DATA.Scale);
    end
    if length(EXP4_DATA.Rtol) == 1 & EXP4_DATA.Rtol ~= 0
      nrm = nrm/EXP4_DATA.Rtol;
    end
end


function [scale] = getscale(varargin)

global EXP4_DATA

if EXP4_DATA.Rtol == 0
  scale = EXP4_DATA.Atol;
elseif length(EXP4_DATA.Rtol) ~= 1 | any(EXP4_DATA.Rtol == 0)
  if nargin == 1
    absval = abs(varargin{1});
  elseif nargin == 2
    absval = max(abs(varargin{1}),abs(varargin{2}));
  end
  scale = EXP4_DATA.Atol + absval.*EXP4_DATA.Rtol;
else
  if nargin == 1
    absval = abs(varargin{1});
  elseif nargin == 2
    absval = max(abs(varargin{1}),abs(varargin{2}));
  end
  scale = EXP4_DATA.Threshold + absval;
end


function [nrm] = current_norm(y)

global EXP4_DATA

if EXP4_DATA.Use2Norm
  nrm = norm(y)/EXP4_DATA.SqrtNeq;
else
  nrm = norm(y,inf);
end
